﻿using Microsoft.Extensions.Logging;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MotorPlantView.Forms
{
    public partial class FormSellEngines : Form
    {
        private readonly ILogger _logger;
        private readonly IEngineLogic _logicP;
        private readonly IShopLogic _logicS;
        private List<EngineViewModel> _EngineList = new List<EngineViewModel>();
        public FormSellEngines(ILogger<FormSellEngines> logger, IEngineLogic logicP, IShopLogic logicS)
        {
            InitializeComponent();
            _logger = logger;
            _logicP = logicP;
            _logicS = logicS;
        }
        private void FormSellingEngine_Load(object sender, EventArgs e)
        {
            _EngineList = _logicP.ReadList(null);
            if (_EngineList != null)
            {
                comboBoxEngine.DisplayMember = "EngineName";
                comboBoxEngine.ValueMember = "Id";
                comboBoxEngine.DataSource = _EngineList;
                comboBoxEngine.SelectedItem = null;
                _logger.LogInformation("Загрузка двигателей для продажи");
            }
        }
        private void ButtonSell_Click(object sender, EventArgs e)
        {
            if (comboBoxEngine.SelectedValue == null)
            {
                MessageBox.Show("Выберите пиццу", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание покупки");
            try
            {
                bool resout = _logicS.Sale(new SupplySearchModel
                {
                    EngineId = Convert.ToInt32(comboBoxEngine.SelectedValue),
                    Count = Convert.ToInt32(textBoxCount.Text)
                });
                if (resout)
                {
                    _logger.LogInformation("Проверка пройдена, продажа  проведена");
                    MessageBox.Show("Продажа проведена", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    DialogResult = DialogResult.OK;
                    Close();
                }
                else
                {
                    _logger.LogInformation("Проверка не пройдена");
                    MessageBox.Show("Продажа не может быть создана.", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания покупки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
